import router, { tempErrorPageName } from '@/router';
import { IdPool } from '@/utils';
import { defineStore } from 'pinia';
import { computed, ref } from 'vue';
import {
  type RouteLocationNormalized,
  type RouteLocationRaw,
  type RouteRecordRaw,
  useRoute
} from 'vue-router';

export type PageErrorReason =
  | {
      type: 'notFound';
    }
  | {
      type: 'networkError';
      originalPath: string;
    }
  | {
      type: 'noPermission';
    };
export const usePageStore = defineStore('page-store', () => {
  const pageErrorReason = ref<PageErrorReason>();
  const routeIdPool = new IdPool();
  const routeIds = routeIdPool.usedIdSet;
  const pageLoadingCount = computed(() => routeIds.size);

  function setNewRouteId(route: RouteLocationNormalized) {
    return (route.meta.routeId = routeIdPool.newId());
  }

  function removeRouteId(route: RouteLocationNormalized) {
    const { routeId } = route.meta;
    if (routeId === undefined) {
      return;
    }
    routeIdPool.removeId(routeId);
  }

  function createTempErrorRoute(
    reason: Exclude<
      PageErrorReason,
      | {
          type: 'notFound';
        }
      | undefined
    >,
    currentRoute: RouteLocationNormalized | undefined = useRoute(),
    root: boolean = false
  ): RouteLocationRaw {
    pageErrorReason.value = reason;
    const tempErrorRoute: RouteRecordRaw = {
      path: currentRoute.fullPath,
      name: tempErrorPageName,
      component: () => import('@/views/ErrorPage.vue')
    };
    const parentRouteName = currentRoute.matched.at(-2)?.name;
    if (parentRouteName !== undefined && !root) {
      router.addRoute(parentRouteName, tempErrorRoute);
    } else {
      router.addRoute(tempErrorRoute);
    }
    return { name: tempErrorPageName, replace: true };
  }

  return {
    pageErrorReason,
    routeIds,
    pageLoadingCount,
    setNewRouteId,
    removeRouteId,
    createTempErrorRoute
  };
});
